@@ -1,4 +1,5 @@
 import argparse
+import datetime
 import getpass
 import json
 import os
@@ -9,12 +10,73 @@ from operator import itemgetter
 from os.path import expanduser
 
 import pkg_resources
+import pygeoj
 import requests
+from bs4 import BeautifulSoup
 from natsort import natsorted
 from tqdm import tqdm
 
-# from bs4 import BeautifulSoup as bs
-# from tenacity import retry, stop_after_attempt, wait_exponential
+
+class Solution:
+    def compareVersion(self, version1, version2):
+        versions1 = [int(v) for v in version1.split(".")]
+        versions2 = [int(v) for v in version2.split(".")]
+        for i in range(max(len(versions1), len(versions2))):
+            v1 = versions1[i] if i < len(versions1) else 0
+            v2 = versions2[i] if i < len(versions2) else 0
+            if v1 > v2:
+                return 1
+            elif v1 < v2:
+                return -1
+        return 0
+
+
+ob1 = Solution()
+
+# Get package version
+
+
+def appeears_version():
+    url = "https://pypi.org/project/appeears/"
+    source = requests.get(url)
+    html_content = source.text
+    soup = BeautifulSoup(html_content, "html.parser")
+    company = soup.find("h1")
+    vcheck = ob1.compareVersion(
+        company.string.strip().split(" ")[-1],
+        pkg_resources.get_distribution("appeears").version,
+    )
+    if vcheck == 1:
+        print(
+            "\n"
+            + "========================================================================="
+        )
+        print(
+            "Current version of pyaqua is {} upgrade to lastest version: {}".format(
+                pkg_resources.get_distribution("appeears").version,
+                company.string.strip().split(" ")[-1],
+            )
+        )
+        print(
+            "========================================================================="
+        )
+    elif vcheck == -1:
+        print(
+            "\n"
+            + "========================================================================="
+        )
+        print(
+            "Possibly running staging code {} compared to pypi release {}".format(
+                pkg_resources.get_distribution("appeears").version,
+                company.string.strip().split(" ")[-1],
+            )
+        )
+        print(
+            "========================================================================="
+        )
+
+
+appeears_version()
 
 error_codes = {
     400: "Bad Request - The request could not be understood by the server due to malformed syntax.",
@@ -124,6 +186,141 @@ def layers_from_parser(args):
     layers(pid=args.pid)
 
 
+# get spatial projection info
+def spatial():
+    response = requests.get(
+        'https://appeears.earthdatacloud.nasa.gov/api/spatial/proj')
+    proj_response = response.json()
+    print(json.dumps(proj_response, indent=2))
+
+
+def spatial_from_parser(args):
+    spatial()
+
+
+# submit a task task_payload
+payload = {
+    'task_type': '',
+    'task_name': '',
+    'params': {
+        'layers': '',
+        'output': {
+            'format': {
+                'type': 'geotiff',
+            },
+            'projection': 'geographic'
+        },
+        'dates': [
+            {
+                'startDate': '',
+                'endDate': '',
+                'recurring': False,
+                'yearRange': [
+                    1950,
+                    2050,
+                ],
+            },
+        ],
+        'coordinates': ''
+    }
+}
+
+
+# submit a task
+def tasksubmit(**kwargs):
+    token = tokenizer()
+    for key, value in kwargs.items():
+        if key == "name":
+            payload["task_name"] = value
+        if key == "input":
+            testfile = pygeoj.load(value)
+            coordinates = []
+            for feature in testfile:
+                if feature.geometry.type == 'Point':
+                    payload["task_type"] = "point"
+                    lat_long_dict = {
+                        "latitude": feature.geometry.coordinates[1],
+                        "longitude": feature.geometry.coordinates[0]
+                    }
+                    coordinates.append(lat_long_dict)
+                    payload['params']['coordinates'] = coordinates
+                elif feature.geometry.type == 'Polygon':
+                    payload["task_type"] = "area"
+                    geom = {"geo": {
+                        'type': 'FeatureCollection',
+                                'features': [
+                                    {
+                                        'type': 'Feature',
+                                        'properties': {},
+                                        'geometry': {
+                                            'type': 'Polygon',
+                                            'coordinates': feature.geometry.coordinates
+                                        },
+                                    },
+                                ]
+                    }, }
+                    coordinates = feature.geometry.coordinates
+                    payload['params'].pop('coordinates')
+                    payload['params'].update(geom)
+                else:
+                    sys.exit("Unknown geometry type")
+
+        if key == "start":
+            value = datetime.datetime.strptime(
+                value, "%Y-%m-%d").strftime("%m-%d-%Y")
+            payload["params"]["dates"][0]["startDate"] = str(value)
+        if key == "end":
+            value = datetime.datetime.strptime(
+                value, "%Y-%m-%d").strftime("%m-%d-%Y")
+            payload["params"]["dates"][0]["endDate"] = str(value)
+        if key == "recurring":
+            payload["params"]["dates"][0]["recurring"] = bool(value)
+        if key == "product":
+            response = requests.get(
+                f"https://appeears.earthdatacloud.nasa.gov/api/product/{value}"
+            )
+            layer_response = response.json()
+            layer_list = []
+            for layer in layer_response:
+                layer_json = {
+                    "layer": layer,
+                    "product": value
+                }
+                layer_list.append(layer_json)
+            payload["params"]["layers"] = layer_list
+    #print(json.dumps(payload, indent=2))
+
+    response = requests.post(
+        'https://appeears.earthdatacloud.nasa.gov/api/task', headers={"Authorization": f"Bearer {token}"}, json=payload)
+    if response.status_code in error_codes.keys():
+        print(error_codes.get(response.status_code))
+        sys.exit(response.json()['message'])
+    else:
+        print(f"Submitted task with task ID {response.json()['task_id']}")
+
+
+def tasksubmit_from_parser(args):
+    tasksubmit(name=args.name, product=args.product, start=args.start,
+               end=args.end, recurring=args.recurring, input=args.geometry)
+
+
+# delete task using task id
+def delete(tid):
+    token = tokenizer()
+    response = requests.delete(
+        f'https://appeears.earthdatacloud.nasa.gov/api/task/{tid}',
+        headers={"Authorization": f"Bearer {token}"})
+    if response.status_code in error_codes.keys():
+        print(error_codes.get(response.status_code))
+        sys.exit(response.json()['message'])
+    elif response.status_code == 204:
+        print(f'Task with task id {tid} deleted')
+
+
+def delete_from_parser(args):
+    delete(tid=args.tid)
+
+
 # get task status for all tasks and groupby status
 def task_all(status):
     token = tokenizer()
@@ -359,8 +556,40 @@ def main(args=None):
     )
     parser_layers.set_defaults(func=layers_from_parser)
 
+    parser_spatial = subparsers.add_parser(
+        "spatial", help="List all supported spatial projections"
+    )
+    parser_spatial.set_defaults(func=spatial_from_parser)
+
+    parser_tasksubmit = subparsers.add_parser(
+        "task-submit", help="Submit your task"
+    )
+    required_named = parser_tasksubmit.add_argument_group(
+        "Required named arguments.")
+    required_named.add_argument(
+        "--name", help="Task name", required=True
+    )
+    required_named.add_argument(
+        "--product", help="Product ID returned from product tool", required=True
+    )
+    required_named.add_argument(
+        "--geometry", help="Full path to geometry.geojson file point or single polygon", required=True
+    )
+    required_named.add_argument(
+        "--start", help="Start date in format YYYY-MM-DD", required=True
+    )
+    required_named.add_argument(
+        "--end", help="End date in format YYYY-MM-DD", required=True
+    )
+    optional_named = parser_tasksubmit.add_argument_group(
+        "Optional named arguments")
+    optional_named.add_argument(
+        "--recurring", help="Date range recurring True|False", default=False
+    )
+    parser_tasksubmit.set_defaults(func=tasksubmit_from_parser)
+
     parser_taskinfo = subparsers.add_parser(
-        "taskinfo",
+        "task-info",
         help="Get task information for all tasks or specific tasks or task status type",
     )
     optional_named = parser_taskinfo.add_argument_group(
@@ -371,6 +600,15 @@ def main(args=None):
     )
     parser_taskinfo.set_defaults(func=taskinfo_from_parser)
 
+    parser_delete = subparsers.add_parser(
+        "delete", help="Delete a specific task with task ID"
+    )
+    required_named = parser_delete.add_argument_group(
+        "Required named arguments.")
+    required_named.add_argument(
+        "--tid", help="Task ID to delete", required=True)
+    parser_delete.set_defaults(func=delete_from_parser)
+
     parser_download = subparsers.add_parser(
         "download", help="Download all files for specific task with task ID"
     )
@@ -391,6 +629,5 @@ def main(args=None):
         parser.error("too few arguments")
     func(args)
 
-
-if __name__ == "__main__":
-    main()
+    if __name__ == "__main__":
+        main()
